/*
 * GroupLens Common Utilities
 * Copyright © 2011 Regents of the University of Minnesota
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *  * Neither the name of the University of Minnesota nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software has been partly funded by NSF grant IIS 08-08692.
 */
package org.grouplens.common.dto;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class DtoContainer<T extends Dto> implements Cloneable {
    private final boolean multiValued;
    private final List<T> contained;
    private final Class<T> dtoType;

    /**
     * Constructs a new single valued, empty Dto container for the given type.
     */
    public DtoContainer(Class<T> type) {
        this(type, false);
    }

    /**
     * Constructs a new single valued Dto container for the given type, with
     * the given initial value.
     */
    public DtoContainer(Class<T> type, T initialValue) {
        this(type, false);
        set(initialValue);
    }

    /**
     * Constructs a new multiple valued Dto container for the given type with
     * the given initial values.
     * @param type
     * @param initialValues
     */
    public DtoContainer(Class<T> type, List<T> initialValues) {
        this(type, true);
        set(initialValues);
    }

    /**
     * @param type the type of Dto to be stored in this.
     * @param multiValued If true, this Dto container is intended to store a
     *        list of values and will only allow list operations on it, otherwise,
     *        this Dto container will only allow singe value operations.
     */
    public DtoContainer(Class<T> type, boolean multiValued) {
        if (type == null)
            throw new NullPointerException("Cannot specify a null type");
        contained = new ArrayList<T>();
        dtoType = type;
        this.multiValued = multiValued;
    }

    /**
     * If this container only allows multiple values an exception will be thrown.
     */
    public T getSingle() {
        assertSingleValued();
        if (contained == null || contained.size() == 0)
            return null;
        else if (contained.size() == 1)
            return contained.get(0);
        else
            throw new IllegalStateException("DtoContainer contains more than 1 Dto");
    }

    /**
     * If this container only allows single values an exception will be thrown.
     * @return The list of Dtos contained within this DtoContainer
     */
    public List<T> get() {
        assertMultipleValued();
        return Collections.unmodifiableList(contained);
    }

    /**
     * Set the contents of this DtoContainer to be the single Dto,
     * <tt>value</tt>. If value is null, the DtoContainer will be empty.
     * If this container only allows multiple values an exception will be thrown.
     * 
     * @param value
     */
    public void set(T value) {
        assertSingleValued();
        contained.clear();
        if (value != null)
            contained.add(value);
    }

    /**
     * Set the contents of this DtoContainer to be the given list of Dtos. If
     * values is null or empty, the DtoContainer will be empty.
     * If this container only allows single values an exception will be thrown.
     */
    public void set(List<T> values) {
        assertMultipleValued();
        contained.clear();
        if (values != null) {
            if (!multiValued && values.size() > 1)
                throw new IllegalStateException("Container does not allow multiple elements");
            contained.addAll(values);
        }
    }

    /**
     * Reset the contents of the DtoContainer
     */
    public void clear() {
        contained.clear();
    }

    public Class<T> getDtoType() {
        return dtoType;
    }

    public boolean isMultiValued() {
        return multiValued;
    }

    private void assertSingleValued() {
        if (multiValued)
            throw new IllegalStateException("Container does not allow single element operations");
    }

    private void assertMultipleValued() {
        if (! multiValued)
            throw new IllegalStateException("Container does not allow multiple element operations");
    }

    @Override
    /**
     * creates a shallow clone of the container.
     */
    public DtoContainer<T> clone() {
        DtoContainer<T> clone = new DtoContainer<T>(dtoType, multiValued);
        if(multiValued) {
            clone.set(get());
        } else {
            clone.set(getSingle());
        }
        return clone;
    }
}